/*
 * self.c
 *
 * This test program illustrates how a process may use the
 * Linux x86 Performance-Monitoring Counters interface to
 * monitor its own execution.
 *
 * The library uses mmap() to map the kernel's accumulated counter
 * state into the process' address space.
 * When perfctr_read_self() is called, it uses the RDPMC and RDTSC
 * instructions to get the current register values, then adds
 * these to the base values found in the mapped-in kernel state.
 * The resulting counts are then delivered to the application.
 *
 * Copyright (C) 1999-2000  Mikael Pettersson
 */
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include "libperfctr.h"

void do_init(void)
{
    struct perfctr_info info;

    if( perfctr_info(&info) < 0 ) {
	perror("perfctr_info");
	exit(1);
    }
    printf("\nPerfCtr Info:\n");
    printf("version_major\t\t%u\n", info.version_major);
    printf("version_minor\t\t%u\n", info.version_minor);
    printf("version_micro\t\t%u\n", info.version_micro);
    printf("nrcpus\t\t\t%u\n", info.nrcpus);
    printf("cpu_type\t\t%u (%s)\n", info.cpu_type, perfctr_cpu_name());
    printf("cpu_features\t\t0x%x\n", info.cpu_features);

    if( perfctr_attach_rdwr_self() < 0 ) {
	perror("perfctr_attach_rdwr_self");
	exit(1);
    }
}

void do_read(void)
{
    struct vperfctr_state state;

    if( perfctr_read_self(&state) < 0 ) {
	 perror("perfctr_read_self");
	 exit(1);
    }
    printf("\nCurrent Sample:\n");
    printf("status\t\t\t%d\n", state.status);
    printf("control.evntsel[0]\t0x%08X\n", state.control.evntsel[0]);
    printf("tsc\t\t\t0x%016llX\n", state.sum.tsc);
    printf("pmc[0]\t\t\t0x%016llX\n", state.sum.pmc[0]);
}

void do_enable(unsigned evntsel0)
{
    struct perfctr_control control;

    memset(&control, 0, sizeof control);
    control.evntsel[0] = evntsel0;
    if( perfctr_control_self(&control) < 0 ) {
	perror("perfctr_control_self");
	exit(1);
    }
}

unsigned fac(unsigned n)
{
    return (n < 2) ? 1 : n * fac(n-1);
}

void do_fac(unsigned n)
{
    printf("\nfac(%u) == %u\n", n, fac(n));
}

int main(void)
{
    do_init();
    do_read();
    do_enable(perfctr_evntsel_num_insns());
    do_fac(15);
    do_read();
    return 0;
}
